@@ -1,5 +1,8 @@
 #include "stdafx.h"
 
+char *last_error = NULL;
+char last_error_b[MAX_ERROR_L] = "";
+
 void print_(const char *s, ...) {
     va_list args;
     va_start(args, s);
@@ -17,13 +20,21 @@ void abort_(const char *s, ...) {
     RUN_ABORT;
 }
 
+void set_last_error_f(char *message, ...) {
+    va_list args;
+    va_start(args, message);
+    vsprintf(last_error_b, message, args);
+    va_end(args);
+    last_error = last_error_b;
+}
+
 ERROR_T read_png(char *file_name, char demultiply, struct pcv_image *image) {
     /* allocates space for some of the simple values that are
     going to be used in the image processing */
     int y;
 
     /* allocates space for the header part of the image so that
-    it must be possible to check for the correct png header */
+    it must be possible to check for the correct PNG header */
     char header[8];
 
     /* allocates space for the pointer to the file that is going to
@@ -41,7 +52,7 @@ ERROR_T read_png(char *file_name, char demultiply, struct pcv_image *image) {
     png_uint_32 buffer_size;
     png_uint_32 row_size;
 
-    /* opens the file and tests for it being a png, this is required
+    /* opens the file and tests for it being a PNG, this is required
     to avoid possible problems while handling inproper files */
 #ifdef _MSC_VER
     wchar_t file_name_w[1024];
@@ -51,27 +62,27 @@ ERROR_T read_png(char *file_name, char demultiply, struct pcv_image *image) {
     fp = fopen(file_name, "rb");
 #endif
     if(!fp) {
-        RAISE_S("[read_png] File %s could not be opened for reading", file_name);
+        RAISE_F("[read_png] File %s could not be opened for reading", file_name);
     }
     count = fread(header, 1, 8, fp);
     if(png_sig_cmp((void *) header, 0, 8) || count != 8) {
-        RAISE_S("[read_png] File %s is not recognized as a PNG file", file_name);
+        RAISE_F("[read_png] File %s is not recognized as a PNG file", file_name);
     }
 
     /* initialize stuff, this is the structu that will be populated
-    withe the complete stat of the png file reading */
+    withe the complete stat of the PNG file reading */
     image->png_ptr = png_create_read_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
     if(!image->png_ptr) {
-        RAISE_S("[read_png] png_create_read_struct failed");
+        RAISE_M("[read_png] png_create_read_struct failed");
     }
 
     image->info_ptr = png_create_info_struct(image->png_ptr);
     if(!image->info_ptr) {
-        RAISE_S("[read_png] png_create_info_struct failed");
+        RAISE_M("[read_png] png_create_info_struct failed");
     }
 
     if(setjmp(png_jmpbuf(image->png_ptr))) {
-        RAISE_S("[read_png] Error during init_io");
+        RAISE_M("[read_png] Error during init_io");
     }
 
     png_init_io(image->png_ptr, fp);
@@ -89,7 +100,7 @@ ERROR_T read_png(char *file_name, char demultiply, struct pcv_image *image) {
     /* reads the complete file value in file, meaning that
     from this point on only decompression is remaining */
     if(setjmp(png_jmpbuf(image->png_ptr))) {
-        RAISE_S("[read_png] Error during read_image");
+        RAISE_M("[read_png] Error during read_image");
     }
 
     /* allocates space in memory for the buffer that will
@@ -132,7 +143,7 @@ ERROR_T write_png_extra(
     FILE *fp;
 
     /* allocates space for temporary pointer values to both the global
-    png file tables and the (meta-)information tables */
+    PNG file tables and the (meta-)information tables */
     png_structp png_ptr;
     png_infop info_ptr;
 
@@ -147,23 +158,23 @@ ERROR_T write_png_extra(
     fp = fopen(file_name, "wb");
 #endif
     if(!fp) {
-        RAISE_S("[write_png] File %s could not be opened for writing", file_name);
+        RAISE_F("[write_png] File %s could not be opened for writing", file_name);
     }
 
     /* initialize stuff of the main structure, so that it may be used
     latter for the write operation */
     png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, NULL, NULL, NULL);
     if(!png_ptr) {
-        RAISE_S("[write_png] png_create_write_struct failed");
+        RAISE_M("[write_png] png_create_write_struct failed");
     }
 
     info_ptr = png_create_info_struct(png_ptr);
     if(!info_ptr) {
-        RAISE_S("[write_png] png_create_info_struct failed");
+        RAISE_M("[write_png] png_create_info_struct failed");
     }
 
     if(setjmp(png_jmpbuf(png_ptr))) {
-        RAISE_S("[write_png] Error during init_io");
+        RAISE_M("[write_png] Error during init_io");
     }
 
     /* in case the multiply mode is defined the image structure
@@ -174,7 +185,7 @@ ERROR_T write_png_extra(
     the error checking verification process */
     png_init_io(png_ptr, fp);
     if(setjmp(png_jmpbuf(png_ptr))) {
-        RAISE_S("[write_png] Error during writing header");
+        RAISE_M("[write_png] Error during writing header");
     }
 
     png_set_IHDR(
@@ -190,27 +201,27 @@ ERROR_T write_png_extra(
     );
 
     /* writes the "header" information described by the
-    info pointer into the png structure */
+    info pointer into the PNG structure */
     png_write_info(png_ptr, info_ptr);
     if(setjmp(png_jmpbuf(png_ptr))) {
-        RAISE_S("[write_png] Error during writing bytes");
+        RAISE_M("[write_png] Error during writing bytes");
     }
 
     png_set_compression_level(png_ptr, compression);
     if(setjmp(png_jmpbuf(png_ptr))) {
-        RAISE_S("[write_png] Error during writing bytes");
+        RAISE_M("[write_png] Error during writing bytes");
     }
 
     png_set_filter(png_ptr, 0, PNG_FILTER_NONE);
     if(setjmp(png_jmpbuf(png_ptr))) {
-        RAISE_S("[write_png] Error during writing bytes");
+        RAISE_M("[write_png] Error during writing bytes");
     }
 
     /* writes the complete set of bytes that are considered
-    to be part of the image into png structure definition */
+    to be part of the image into PNG structure definition */
     png_write_image(png_ptr, image->rows);
     if(setjmp(png_jmpbuf(png_ptr))) {
-        RAISE_S("[write_png] Error during end of write");
+        RAISE_M("[write_png] Error during end of write");
     }
 
     png_write_end(png_ptr, NULL);
@@ -280,14 +291,14 @@ ERROR_T process_image(struct pcv_image *image) {
     int y;
 
     if(png_get_color_type(image->png_ptr, image->info_ptr) == PNG_COLOR_TYPE_RGB) {
-        RAISE_S(
+        RAISE_M(
             "[process_image] input file is PNG_COLOR_TYPE_RGB but must be PNG_COLOR_TYPE_RGBA "
             "(lacks the alpha channel)"
         );
     }
 
     if(png_get_color_type(image->png_ptr, image->info_ptr) != PNG_COLOR_TYPE_RGBA) {
-        RAISE_S(
+        RAISE_F(
             "[process_image] color_type of input file must be PNG_COLOR_TYPE_RGBA (%d) (is %d)",
             PNG_COLOR_TYPE_RGBA,
             png_get_color_type(image->png_ptr, image->info_ptr)
@@ -669,8 +680,8 @@ ERROR_T compose_images_extra(
     NORMAL;
 }
 
-int pcompose(int argc, char **argv) {
-    if(argc != 3) { abort_("Usage: pconvert compose <directory>"); }
+ERROR_T pcompose(int argc, char **argv) {
+    if(argc != 3) { RAISE_M("Usage: pconvert compose <directory>"); }
 
     compose_images(argv[2], "alpha", NULL, "alpha");
     compose_images(argv[2], "alpha", NULL, "white");
@@ -709,20 +720,20 @@ int pcompose(int argc, char **argv) {
     compose_images(argv[2], "disjoint_debug", NULL, "blue");
     compose_images(argv[2], "disjoint_debug", NULL, "texture");
 
-    return 0;
+    NORMAL;
 }
 
-int pconvert(int argc, char **argv) {
+ERROR_T pconvert(int argc, char **argv) {
     struct pcv_image image;
 
-    if(argc != 4) { abort_("Usage: pconvert convert <file_in> <file_out>"); }
+    if(argc != 4) { RAISE_M("Usage: pconvert convert <file_in> <file_out>"); }
 
     read_png(argv[2], FALSE, &image);
     process_image(&image);
     write_png(&image, FALSE, argv[3]);
     release_image(&image);
 
-    return 0;
+    NORMAL;
 }
 
 float pbenchmark_algorithm(
@@ -743,11 +754,11 @@ float pbenchmark_algorithm(
     return time_elapsed;
 }
 
-int pbenchmark(int argc, char **argv) {
+ERROR_T pbenchmark(int argc, char **argv) {
     float time;
     FILE *file;
 
-    if(argc != 3) { abort_("Usage: pconvert benchmark <directory>"); }
+    if(argc != 3) { RAISE_M("Usage: pconvert benchmark <directory>"); }
 
     file = fopen("benchmark.txt", "wb");
 
@@ -773,16 +784,16 @@ int pbenchmark(int argc, char **argv) {
 
     fclose(file);
 
-    return 0;
+    NORMAL;
 }
 
-int popencl(int argc, char **argv) {
+ERROR_T popencl(int argc, char **argv) {
     float start_time;
     float end_time;
     float time_elapsed_cpu;
     float time_elapsed_gpu;
 
-    if(argc != 3) { abort_("Usage: pconvert opencl <directory>"); }
+    if(argc != 3) { RAISE_M("Usage: pconvert opencl <directory>"); }
 
     time_elapsed_cpu = 0;
     start_time = (float) clock() / CLOCKS_PER_SEC;
@@ -810,10 +821,25 @@ int popencl(int argc, char **argv) {
 }
 
 int main(int argc, char **argv) {
-    if(argc < 2) { abort_("Usage: pconvert <command> [args...]"); }
-    if(strcmp(argv[1], "compose") == 0) { return pcompose(argc, argv); }
-    else if(strcmp(argv[1], "convert") == 0) { return pconvert(argc, argv); }
-    else if(strcmp(argv[1], "benchmark") == 0) { return pbenchmark(argc, argv); }
-    else if(strcmp(argv[1], "opencl") == 0) { return popencl(argc, argv); }
-    else { abort_("Usage: pconvert <command> [args...]"); return 0; }
+    set_last_error("%s", "tobias");
+
+    if(argc < 2) {
+        abort_("Usage: pconvert <command> [args...]");
+        return ERROR;
+    }
+
+    if(strcmp(argv[1], "compose") == 0) {
+        EXCEPT_S(pcompose(argc, argv));
+    } else if(strcmp(argv[1], "convert") == 0) {
+        EXCEPT_S(pconvert(argc, argv));
+    } else if(strcmp(argv[1], "benchmark") == 0) {
+        EXCEPT_S(pbenchmark(argc, argv));
+    } else if(strcmp(argv[1], "opencl") == 0) {
+        EXCEPT_S(popencl(argc, argv));
+    } else {
+        abort_("Usage: pconvert <command> [args...]");
+        return ERROR;
+    }
+
+    return NO_ERROR;
 }
